<?php
/**
 * @file
 * DOM based implementation of the OAI-PMH parser.
 * 
 * The XML response is parsed with Document Object Model (DOM) methods. It is 
 * a wrapper of the Omeka classes.
 * 
 * @author Király Péter <kirunews@gmail.com>
 * 
 * @copyright (c) 2010-2011 eXtensible Catalog Organization
 */

class OAIDomParser implements IOAIParser {

  private $oai;
  private $recordCounter = -1;
  private $recordCount   = -1;
  private $httpCode;
  private $httpErrorMsg;
  private $statistics;

  function __construct($content, $httpCode = 200) {
    $t0 = microtime(TRUE);
    $oaiDom = new SimpleXMLIterator($content);
    $this->statistics['response'] = microtime(TRUE) - $t0;
    $this->oai = $oaiDom;
    $this->httpCode = $httpCode;
  }

  function hasErrors() {
    return isset($this->oai->error) || ((int)$this->httpCode != 200);
  }

  function listErrors() {
    return $this->oai->error;
  }
  
  function getNextRecord() {
    $this->recordCounter++;
    if ($this->recordCounter < $this->getRecordCount()) {
      $t0 = microtime(TRUE);
      $record = $this->parseRecord($this->oai->ListRecords->record[$this->recordCounter]);
      $this->statistics['records'] += (microtime(TRUE) - $t0);
      return $record;
    }
    return FALSE;
  }

  function hasRecords() {
    return $this->getRecordCount() > 0;
  }
  
  function getRecordCount() {
    if ($this->recordCount == -1) {
       $this->recordCount = $this->oai->ListRecords->record->count();
    }
    return $this->recordCount;
  }

  function hasResumptionToken() {
    $token = (string)$this->oai->ListRecords->resumptionToken;
    return !empty($token);
  }

  /**
   * Return the resumption token with its attributes
   * @code
   * array ( 
   *   'attributes' => array ( ), 
   *   'text'       => '100/Jm1ldGFkYXRhUHJlZml4PW9haV9kYyZz', 
   * )
   * /@code
   * 
   * @see oaiharvester/includes/IOAIParser#get_resumptionToken()
   */
  function getResumptionToken() {
    $token = $this->oai->ListRecords->resumptionToken;
    return array(
      'attributes' => array(
        'completeListSize' => (int)$token->attributes()->completeListSize,
        'cursor'           => (int)$token->attributes()->cursor,
        'expirationDate'   => (string)$token->attributes()->expirationDate,
      ),
      'text' => (string)$token
    );
  }
  
  function setHttpCode($httpCode) {
    $this->httpCode = $httpCode;
  }

  function getHttpCode() {
    return $this->httpCode;
  }
  
  function parseRecord($record) {
    $specs = array();
    foreach ($record->header->setSpec as $spec) {
      $specs[] = (string)$spec;
    }
    $dom = dom_import_simplexml($record->metadata);
    foreach ($dom->childNodes as $childNode) {
      if ($childNode->nodeType == XML_ELEMENT_NODE) {
        $metadata = array(
          'namespaceURI' => preg_replace('/\/$/', '', $childNode->namespaceURI),
          'childNode'    => $childNode,
        );
        break;
      }
    }
    
    return array(
      'header' => array(
        '@status' => isset($record->header->attributes()->status) ? $record->header->attributes()->status : FALSE,
        'identifier' => (string)$record->header->identifier[0], 
        'datestamp' => (string)$record->header->datestamp[0], 
        'setSpec' => $specs
      ),
      'metadata' => $metadata, 
      'about' => isset($record->abouts) ? $record->abouts : FALSE
    );
  }
  
  function getStatistics($type = 'all') {
    switch ($type) {
      case 'all':
        return $this->statistics; break;
      case 'response':
        return $this->statistics['response'];
      case 'records':
        return $this->statistics['records'];
      default:
        return;
    }
  }
  
}