<?php
/**
 * @file
 * XC Metadata statistics functions
 * 
 * It provides information about the number of records in the Drupal Toolkit in three
 * different time: at the pre-harvest state, during harvesting, and in the current state.
 * 
 * The statistics of current state is created on demand. The previous statistics 
 * are saved in Drupal variables 'xc_metadata_stat_previous', and 'xc_metadata_stat_harvested'.
 * They are collected during harvest in xc_oaiharvester_bridge module's implementation
 * of different OAI Harvester hooks. The statistics of harvesting is collected into
 * the globa variablel $_xc_metadata_statistics.
 * 
 * @see xc_oaiharvester_bridge_oaiharvester_harvest_starting
 * @see xc_oaiharvester_bridge_oaiharvester_request_started
 * @see xc_oaiharvester_bridge_oaiharvester_process_record
 * @see xc_oaiharvester_bridge_oaiharvester_request_processed
 * 
 * @copyright (c) 2010-2011 eXtensible Catalog Organization
 */

/**
 * View the metadata statistics.
 * 
 * It provides information about pre-harvest state, the harvested records, and the current state
 * of metadata in Drupal Toolkit
 */
function xc_metadata_statistics_view() {
  $states = array(
    'previous' => xc_metadata_statistics('previous'),
    'harvested' => xc_metadata_statistics('harvested'),
    'current' => xc_metadata_statistics('current'),
  );
  $properties = array(
    t('All entities') => 'entities',
    t('Work') => 'work',
    t('Expression') => 'expression',
    t('Manifestation') => 'manifestation',
    t('Holdings') => 'holdings',
    t('Dublin Core') => 'oai_dc',
    t('Metadata') => 'metadata',
    t('Nodes') => 'nodes',
    t('Relations') => 'relations',
    t('New') => 'new',
    t('Updated') => 'updated',
    t('Deleted') => 'deleted',
  );
  
  $rows = array();
  foreach ($properties as $label => $property) {
    $row = array($label);
    foreach ($states as $state_name => $state) {
      $style = FALSE;
      $cell = isset($state->$property) ? $state->$property : t('n/a');
      if ($property == 'entities' && isset($state->work)) {
        $style = ($state->entities == ($state->work + $state->expression + $state->manifestation + $state->holdings))
          ? 'color: green' : 'color: red';
      }
      elseif ($property == 'metadata' && $state_name != 'harvested' && isset($state->metadata)) {
        $style = ($state->metadata == $state->entities) ? 'color: green' : 'color: red';
      }
      elseif ($property == 'nodes' && $state_name != 'harvested' && isset($state->manifestation)) {
        $style = ($state->nodes == $state->manifestation) ? 'color: green' : 'color: red';
      }
      if ($style) {
        $row[] = array('data' => $cell, 'style' => $style);
      }
      else {
        $row[] = $cell;
      }
    }
    $rows[] = $row;
  }
  
  $headers = array(t('Properties'), t('Before harvest'), t('Harvested'), t('Current'));
  return theme('table', $headers, $rows);
}

/**
 * Collects and returns statistical information about metadata
 * 
 * @param string $type
 *   The type of statistical information. Possible values are
 *   - previous: the pre-harvest state's statistics
 *   - harvested: the statistics about harvested records
 *   - current: the current state's statistics
 *   
 * @return Object
 *   A stdClass object contains pairs of properties and counts belonging to properties.
 *   The keys are:
 *   - entities: the number of entities
 *   - oai_dc: the number of Dublin Core records
 *   - work:  the number of FRBR work records
 *   - expression: the number of FRBR expression records
 *   - manifestation: the number of FRBR manifestation records
 *   - holdings: the number of FRBR holdings records
 *   - metadata: the number of actual metadata part of any type of entities
 *   - nodes: the number of nodes created
 *   - relations: the number of relations between entities
 *   - new: the number of new records
 *   - updated: the number of updated records
 *   - deleted: the number of deleted records
 *   
 * @see xc_metadata_statistics_empty
 */
function xc_metadata_statistics($type = 'current') {
  switch ($type) {
    case 'previous':
      $statistics = variable_get('xc_metadata_stat_previous', xc_metadata_statistics_empty());
      break;
    
    case 'harvested':
      $statistics = variable_get('xc_metadata_stat_harvested', xc_metadata_statistics_empty());
      break;
      
    case 'current':
      $sql = 'SELECT 
            (SELECT count(*) FROM {xc_entity_properties}) AS entities, 
            (SELECT count(*) FROM {xc_sql_metadata}) AS metadata,
            (SELECT count(*) FROM {node}) AS nodes, 
            (SELECT count(*) FROM {xc_entity_relationships}) AS relations';
      $statistics = db_fetch_object(db_query($sql));
      $result = db_query('SELECT metadata_type, COUNT(metadata_type) AS count 
                          FROM {xc_entity_properties} GROUP BY metadata_type');
      $statistics->oai_dc = 0;
      $statistics->work = 0;
      $statistics->expression = 0;
      $statistics->manifestation = 0;
      $statistics->holdings = 0;
      while ($data = db_fetch_object($result)) {
        $statistics->{$data->metadata_type} = $data->count;
      }
      
      break;

    default:
      $statistics = variable_get('xc_metadata_stat_previous', xc_metadata_statistics_empty());
      break;
  }
  return $statistics;
}

/**
 * Create an empty statistics object
 * 
 * @return Object
 *   A stdClass object contains pairs of properties and counts belonging to properties.
 *   Each count is 0 in this initial state of the object
 *   The keys are:
 *   - entities: the number of entities
 *   - oai_dc: the number of Dublin Core records
 *   - work:  the number of FRBR work records
 *   - expression: the number of FRBR expression records
 *   - manifestation: the number of FRBR manifestation records
 *   - holdings: the number of FRBR holdings records
 *   - metadata: the number of actual metadata part of any type of entities
 *   - nodes: the number of nodes created
 *   - relations: the number of relations between entities
 *   - new: the number of new records
 *   - updated: the number of updated records
 *   - deleted: the number of deleted records
 */
function xc_metadata_statistics_empty() {
  $statistics = (Object)array(
    'entities' => 0,
    'oai_dc' => 0,
    'work' => 0,
    'expression' => 0,
    'manifestation' => 0,
    'holdings' => 0,
    'metadata' => 0,
    'nodes' => 0,
    'relations' => 0,
    'new' => 0,
    'updated' => 0,
    'deleted' => 0,
  );
  return $statistics;
}
